"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var fs_1 = __importDefault(require("fs"));
var typescript_1 = __importDefault(require("typescript"));
var createDefaultFilesIfNotExists_1 = require("./createDefaultFilesIfNotExists");
var findRootFiles = function (dir) {
    return fs_1.default
        .readdirSync(dir, { withFileTypes: true })
        .reduce(function (prev, d) { return __spreadArray(__spreadArray([], __read(prev), false), __read((d.isDirectory()
        ? findRootFiles("".concat(dir, "/").concat(d.name))
        : d.name === 'hooks.ts' || d.name === 'controller.ts'
            ? ["".concat(dir, "/").concat(d.name)]
            : [])), false); }, []);
};
var initTSC = function (appDir, project) {
    var configDir = path_1.default.resolve(project.replace(/\/[^/]+\.json$/, ''));
    var configFileName = typescript_1.default.findConfigFile(configDir, typescript_1.default.sys.fileExists, project.endsWith('.json') ? project.split('/').pop() : undefined);
    var compilerOptions = configFileName
        ? typescript_1.default.parseJsonConfigFileContent(typescript_1.default.readConfigFile(configFileName, typescript_1.default.sys.readFile).config, typescript_1.default.sys, configDir)
        : undefined;
    var program = typescript_1.default.createProgram(findRootFiles(appDir), (compilerOptions === null || compilerOptions === void 0 ? void 0 : compilerOptions.options)
        ? { baseUrl: compilerOptions === null || compilerOptions === void 0 ? void 0 : compilerOptions.options.baseUrl, paths: compilerOptions === null || compilerOptions === void 0 ? void 0 : compilerOptions.options.paths }
        : {});
    return { program: program, checker: program.getTypeChecker() };
};
var createRelayFile = function (input, appText, additionalReqs, params, currentParam) {
    var hasAdditionals = !!additionalReqs.length;
    var hasMultiAdditionals = additionalReqs.length > 1;
    var text = "".concat(currentParam ? "import { z } from 'zod'\n" : '', "import type { Injectable } from 'velona'\nimport { depend } from 'velona'\nimport type { FastifyInstance, onRequestHookHandler, preParsingHookHandler, preValidationHookHandler, preHandlerHookHandler } from 'fastify'\nimport type { Schema } from 'fast-json-stringify'\nimport type { HttpStatusOk } from 'aspida'\nimport type { ServerMethodHandler } from '").concat(appText, "'\n").concat(hasMultiAdditionals
        ? additionalReqs
            .map(function (req, i) {
            return "import type { AdditionalRequest as AdditionalRequest".concat(i, " } from '").concat(req.replace(/^\.\/\./, '.'), "'\n");
        })
            .join('')
        : hasAdditionals
            ? "import type { AdditionalRequest } from '".concat(additionalReqs[0], "'\n")
            : '', "import type { Methods } from './'\n\n").concat(hasMultiAdditionals
        ? "type AdditionalRequest = ".concat(additionalReqs
            .map(function (_, i) { return "AdditionalRequest".concat(i); })
            .join(' & '), "\n")
        : '').concat(hasAdditionals
        ? 'type AddedHandler<T> = T extends (req: infer U, ...args: infer V) => infer W ? (req: U & Partial<AdditionalRequest>, ...args: V) => W : never\n'
        : '', "type Hooks = {\n").concat([
        ['onRequest', 'onRequestHookHandler'],
        ['preParsing', 'preParsingHookHandler'],
        ['preValidation', 'preValidationHookHandler'],
        ['preHandler', 'preHandlerHookHandler']
    ]
        .map(function (_a) {
        var _b = __read(_a, 2), key = _b[0], val = _b[1];
        return hasAdditionals
            ? "  ".concat(key, "?: AddedHandler<").concat(val, "> | AddedHandler<").concat(val, ">[]\n")
            : "  ".concat(key, "?: ").concat(val, " | ").concat(val, "[]\n");
    })
        .join(''), "}").concat(params.length ? "\ntype Params = {\n".concat(params.map(function (v) { return "  ".concat(v[0], ": ").concat(v[1]); }).join('\n'), "\n}") : '').concat(currentParam
        ? "\n\nexport function defineValidators(validator: (fastify: FastifyInstance) => {\n  params: z.ZodType<{ ".concat(currentParam[0], ": ").concat(currentParam[1], " }>\n}) {\n  return validator\n}")
        : '', "\n\nexport function defineResponseSchema<T extends { [U in keyof Methods]?: { [V in HttpStatusOk]?: Schema }}>(methods: () => T) {\n  return methods\n}\n\nexport function defineHooks<T extends Hooks>(hooks: (fastify: FastifyInstance) => T): (fastify: FastifyInstance) => T\nexport function defineHooks<T extends Record<string, any>, U extends Hooks>(deps: T, cb: (d: T, fastify: FastifyInstance) => U): Injectable<T, [FastifyInstance], U>\nexport function defineHooks<T extends Record<string, any>>(hooks: (fastify: FastifyInstance) => Hooks | T, cb?: ((deps: T, fastify: FastifyInstance) => Hooks)) {\n  return cb && typeof hooks !== 'function' ? depend(hooks, cb) : hooks\n}\n\ntype ServerMethods = {\n  [Key in keyof Methods]: ServerMethodHandler<Methods[Key]").concat(hasAdditionals || params.length ? ', ' : '').concat(hasAdditionals ? "AdditionalRequest".concat(params.length ? ' & ' : '') : '').concat(params.length ? '{ params: Params }' : '', ">\n}\n\nexport function defineController<M extends ServerMethods>(methods: (fastify: FastifyInstance) => M): (fastify: FastifyInstance) => M\nexport function defineController<M extends ServerMethods, T extends Record<string, any>>(deps: T, cb: (d: T, fastify: FastifyInstance) => M): Injectable<T, [FastifyInstance], M>\nexport function defineController<M extends ServerMethods, T extends Record<string, any>>(methods: ((fastify: FastifyInstance) => M) | T, cb?: ((deps: T, fastify: FastifyInstance) => M)) {\n  return cb && typeof methods !== 'function' ? depend(methods, cb) : methods\n}\n");
    fs_1.default.writeFileSync(path_1.default.join(input, '$relay.ts'), text.replace(', {}', '').replace(' & {}', ''), 'utf8');
};
var getAdditionalResPath = function (input, name) {
    return fs_1.default.existsSync(path_1.default.join(input, "".concat(name, ".ts"))) &&
        /(^|\n)export .+ AdditionalRequest(,| )/.test(fs_1.default.readFileSync(path_1.default.join(input, "".concat(name, ".ts")), 'utf8'))
        ? ["./".concat(name)]
        : [];
};
var createFiles = function (appDir, dirPath, params, currentParam, appPath, additionalRequestPaths) {
    var input = path_1.default.posix.join(appDir, dirPath);
    var appText = "../".concat(appPath);
    var additionalReqs = __spreadArray(__spreadArray([], __read(additionalRequestPaths.map(function (p) { return "./.".concat(p); })), false), __read(getAdditionalResPath(input, 'hooks')), false);
    (0, createDefaultFilesIfNotExists_1.createDefaultFilesIfNotExists)(input, currentParam);
    createRelayFile(input, appText, __spreadArray(__spreadArray([], __read(additionalReqs), false), __read(getAdditionalResPath(input, 'controller')), false), params, currentParam);
    var dirs = fs_1.default.readdirSync(input, { withFileTypes: true }).filter(function (d) { return d.isDirectory(); });
    if (dirs.filter(function (d) { return d.name.startsWith('_'); }).length >= 2) {
        throw new Error('There are two ore more path param folders.');
    }
    dirs.forEach(function (d) {
        var _a;
        var currentParam = d.name.startsWith('_')
            ? [d.name.slice(1).split('@')[0], (_a = d.name.split('@')[1]) !== null && _a !== void 0 ? _a : 'string']
            : null;
        return createFiles(appDir, path_1.default.posix.join(dirPath, d.name), currentParam ? __spreadArray(__spreadArray([], __read(params), false), [currentParam], false) : params, currentParam, appText, additionalReqs);
    });
};
exports.default = (function (appDir, project) {
    createFiles(appDir, '', [], null, '$server', []);
    var _a = initTSC(appDir, project), program = _a.program, checker = _a.checker;
    var hooksPaths = [];
    var validatorsPaths = [];
    var controllers = [];
    var createText = function (dirPath, cascadingHooks, cascadingValidators) {
        var input = path_1.default.posix.join(appDir, dirPath);
        var source = program.getSourceFile(path_1.default.join(input, 'index.ts'));
        var results = [];
        var hooks = cascadingHooks;
        var paramsValidators = cascadingValidators;
        var validatorsFilePath = path_1.default.join(input, 'validators.ts');
        if (fs_1.default.existsSync(validatorsFilePath)) {
            paramsValidators = __spreadArray(__spreadArray([], __read(cascadingValidators), false), [
                {
                    name: "validators".concat(validatorsPaths.length),
                    isNumber: dirPath.split('@')[1] === 'number'
                }
            ], false);
            validatorsPaths.push("".concat(input, "/validators"));
        }
        if (source) {
            var methods = typescript_1.default.forEachChild(source, function (node) {
                var _a;
                return (typescript_1.default.isTypeAliasDeclaration(node) || typescript_1.default.isInterfaceDeclaration(node)) &&
                    node.name.escapedText === 'Methods' &&
                    ((_a = node.modifiers) === null || _a === void 0 ? void 0 : _a.some(function (m) { return m.kind === typescript_1.default.SyntaxKind.ExportKeyword; }))
                    ? checker.getTypeAtLocation(node).getProperties()
                    : undefined;
            });
            var hooksSource = program.getSourceFile(path_1.default.join(input, 'hooks.ts'));
            if (hooksSource) {
                var events = typescript_1.default.forEachChild(hooksSource, function (node) {
                    if (typescript_1.default.isExportAssignment(node)) {
                        return node.forEachChild(function (node) {
                            return typescript_1.default.isCallExpression(node) &&
                                node.forEachChild(function (node) {
                                    if (typescript_1.default.isMethodDeclaration(node) ||
                                        typescript_1.default.isArrowFunction(node) ||
                                        typescript_1.default.isFunctionDeclaration(node)) {
                                        return (node.body &&
                                            checker
                                                .getTypeAtLocation(node.body)
                                                .getProperties()
                                                .map(function (p) {
                                                var typeNode = p.valueDeclaration &&
                                                    checker.typeToTypeNode(checker.getTypeOfSymbolAtLocation(p, p.valueDeclaration), undefined, undefined);
                                                return {
                                                    type: p.name,
                                                    isArray: typeNode
                                                        ? typescript_1.default.isArrayTypeNode(typeNode) || typescript_1.default.isTupleTypeNode(typeNode)
                                                        : false
                                                };
                                            }));
                                    }
                                });
                        });
                    }
                });
                if (events) {
                    hooks = __spreadArray(__spreadArray([], __read(cascadingHooks), false), [{ name: "hooks".concat(hooksPaths.length), events: events }], false);
                    hooksPaths.push("".concat(input, "/hooks"));
                }
            }
            if (methods === null || methods === void 0 ? void 0 : methods.length) {
                var controllerSource_1 = program.getSourceFile(path_1.default.join(input, 'controller.ts'));
                var isPromiseMethods_1 = [];
                var hasHandlerMethods_1 = [];
                var hasValidatorsMethods_1 = [];
                var hasSchemasMethods_1 = [];
                var hasHooksMethods_1 = [];
                var ctrlHooksSignature = void 0;
                var resSchemaSignature = void 0;
                if (controllerSource_1) {
                    var getMethodTypeNodes = function (cb) {
                        return typescript_1.default.forEachChild(controllerSource_1, function (node) {
                            return typescript_1.default.isExportAssignment(node) &&
                                node.forEachChild(function (nod) {
                                    return typescript_1.default.isCallExpression(nod) &&
                                        checker
                                            .getSignaturesOfType(checker.getTypeAtLocation(nod.arguments[nod.arguments.length - 1]), typescript_1.default.SignatureKind.Call)[0]
                                            .getReturnType()
                                            .getProperties()
                                            .map(function (t) {
                                            var type = t.valueDeclaration &&
                                                checker.getTypeOfSymbolAtLocation(t, t.valueDeclaration);
                                            if (!type)
                                                return undefined;
                                            var typeNode = t.valueDeclaration && checker.typeToTypeNode(type, undefined, undefined);
                                            if (!typeNode)
                                                return undefined;
                                            return cb(t, typeNode, type);
                                        })
                                            .filter(function (n) { return !!n; });
                                });
                        }) || [];
                    };
                    isPromiseMethods_1.push.apply(isPromiseMethods_1, __spreadArray([], __read(getMethodTypeNodes(function (symbol, typeNode, type) {
                        var _a;
                        var handler = typescript_1.default.isFunctionTypeNode(typeNode)
                            ? symbol
                            : type.getProperties().find(function (p) { return p.name === 'handler'; });
                        if (!handler)
                            return null;
                        return handler.valueDeclaration &&
                            ((_a = checker
                                .getSignaturesOfType(checker.getTypeOfSymbolAtLocation(handler, handler.valueDeclaration), typescript_1.default.SignatureKind.Call)[0]
                                .getReturnType()
                                .getSymbol()) === null || _a === void 0 ? void 0 : _a.getEscapedName()) === 'Promise'
                            ? symbol.name
                            : null;
                    })), false));
                    hasHandlerMethods_1.push.apply(hasHandlerMethods_1, __spreadArray([], __read(getMethodTypeNodes(function (symbol, typeNode) {
                        return typescript_1.default.isFunctionTypeNode(typeNode) ? null : symbol.name;
                    })), false));
                    hasValidatorsMethods_1.push.apply(hasValidatorsMethods_1, __spreadArray([], __read(getMethodTypeNodes(function (symbol, typeNode, type) {
                        return !typescript_1.default.isFunctionTypeNode(typeNode) &&
                            type.getProperties().find(function (p) { return p.name === 'validators'; })
                            ? symbol.name
                            : null;
                    })), false));
                    hasSchemasMethods_1.push.apply(hasSchemasMethods_1, __spreadArray([], __read(getMethodTypeNodes(function (symbol, typeNode, type) {
                        return !typescript_1.default.isFunctionTypeNode(typeNode) &&
                            type.getProperties().find(function (p) { return p.name === 'schemas'; })
                            ? symbol.name
                            : null;
                    })), false));
                    hasHooksMethods_1.push.apply(hasHooksMethods_1, __spreadArray([], __read(getMethodTypeNodes(function (symbol, typeNode, type) {
                        if (typescript_1.default.isFunctionTypeNode(typeNode))
                            return null;
                        var hooksSymbol = type.getProperties().find(function (p) { return p.name === 'hooks'; });
                        if (!(hooksSymbol === null || hooksSymbol === void 0 ? void 0 : hooksSymbol.valueDeclaration))
                            return null;
                        return {
                            method: symbol.name,
                            events: checker
                                .getTypeOfSymbolAtLocation(hooksSymbol, hooksSymbol.valueDeclaration)
                                .getProperties()
                                .map(function (p) {
                                var typeNode = p.valueDeclaration &&
                                    checker.typeToTypeNode(checker.getTypeOfSymbolAtLocation(p, p.valueDeclaration), undefined, undefined);
                                return {
                                    type: p.name,
                                    isArray: typeNode
                                        ? typescript_1.default.isArrayTypeNode(typeNode) || typescript_1.default.isTupleTypeNode(typeNode)
                                        : false
                                };
                            })
                        };
                    })), false));
                    var ctrlHooksNode_1;
                    var resSchemaNode_1;
                    typescript_1.default.forEachChild(controllerSource_1, function (node) {
                        var _a, _b, _c, _d, _e;
                        if (typescript_1.default.isVariableStatement(node) &&
                            ((_a = node.modifiers) === null || _a === void 0 ? void 0 : _a.some(function (m) { return m.kind === typescript_1.default.SyntaxKind.ExportKeyword; }))) {
                            ctrlHooksNode_1 =
                                (_b = node.declarationList.declarations.find(function (d) { return d.name.getText() === 'hooks'; })) !== null && _b !== void 0 ? _b : ctrlHooksNode_1;
                            resSchemaNode_1 =
                                (_c = node.declarationList.declarations.find(function (d) { return d.name.getText() === 'responseSchema'; })) !== null && _c !== void 0 ? _c : resSchemaNode_1;
                        }
                        else if (typescript_1.default.isExportDeclaration(node)) {
                            var exportClause = node.exportClause;
                            if (exportClause && typescript_1.default.isNamedExports(exportClause)) {
                                ctrlHooksNode_1 =
                                    (_d = exportClause.elements.find(function (el) { return el.name.text === 'hooks'; })) !== null && _d !== void 0 ? _d : ctrlHooksNode_1;
                                resSchemaNode_1 =
                                    (_e = exportClause.elements.find(function (el) { return el.name.text === 'responseSchema'; })) !== null && _e !== void 0 ? _e : resSchemaNode_1;
                            }
                        }
                    });
                    if (ctrlHooksNode_1) {
                        ctrlHooksSignature = checker.getSignaturesOfType(checker.getTypeAtLocation(ctrlHooksNode_1), typescript_1.default.SignatureKind.Call)[0];
                    }
                    if (resSchemaNode_1) {
                        resSchemaSignature = checker.getSignaturesOfType(checker.getTypeAtLocation(resSchemaNode_1), typescript_1.default.SignatureKind.Call)[0];
                    }
                }
                var ctrlHooksEvents_1 = ctrlHooksSignature === null || ctrlHooksSignature === void 0 ? void 0 : ctrlHooksSignature.getReturnType().getProperties().map(function (p) {
                    var typeNode = p.valueDeclaration &&
                        checker.typeToTypeNode(checker.getTypeOfSymbolAtLocation(p, p.valueDeclaration), undefined, undefined);
                    return {
                        type: p.name,
                        isArray: typeNode
                            ? typescript_1.default.isArrayTypeNode(typeNode) || typescript_1.default.isTupleTypeNode(typeNode)
                            : false
                    };
                });
                var genHookTexts_1 = function (event, methodName) {
                    var _a, _b, _c;
                    return __spreadArray(__spreadArray(__spreadArray([], __read(hooks.reduce(function (prev, h) {
                        var ev = h.events.find(function (e) { return e.type === event; });
                        return ev ? __spreadArray(__spreadArray([], __read(prev), false), ["".concat(ev.isArray ? '...' : '').concat(h.name, ".").concat(event)], false) : prev;
                    }, [])), false), __read(((_a = ctrlHooksEvents_1 === null || ctrlHooksEvents_1 === void 0 ? void 0 : ctrlHooksEvents_1.map(function (e) {
                        return e.type === event
                            ? "".concat(e.isArray ? '...' : '', "ctrlHooks").concat(controllers.filter(function (c) { return c[1]; }).length, ".").concat(event)
                            : '';
                    })) !== null && _a !== void 0 ? _a : [])), false), __read((hasHooksMethods_1.some(function (m) { return m.method === methodName && m.events.some(function (e) { return e.type === event; }); })
                        ? [
                            "".concat(((_c = (_b = hasHooksMethods_1
                                .find(function (m) { return m.method === methodName; })) === null || _b === void 0 ? void 0 : _b.events.find(function (e) { return e.type === event; })) === null || _c === void 0 ? void 0 : _c.isArray)
                                ? '...'
                                : '', "controller").concat(controllers.length, ".").concat(methodName, ".hooks.").concat(event)
                        ]
                        : [])), false);
                };
                var resSchemaMethods_1 = resSchemaSignature === null || resSchemaSignature === void 0 ? void 0 : resSchemaSignature.getReturnType().getProperties().map(function (p) { return p.name; });
                var genResSchemaText_1 = function (method) {
                    return "response: responseSchema".concat(controllers.filter(function (c) { return c[2]; }).length, ".").concat(method);
                };
                var getSomeTypeQueryParams_1 = function (typeName, query) {
                    return query.valueDeclaration &&
                        checker
                            .getTypeOfSymbolAtLocation(query, query.valueDeclaration)
                            .getProperties()
                            .map(function (p) {
                            var _a;
                            var typeString = p.valueDeclaration &&
                                checker.typeToString(checker.getTypeOfSymbolAtLocation(p, p.valueDeclaration));
                            return typeString === typeName || typeString === "".concat(typeName, "[]")
                                ? "['".concat(p.name, "', ").concat(!!((_a = p.declarations) === null || _a === void 0 ? void 0 : _a.some(function (d) {
                                    return d.getChildren().some(function (c) { return c.kind === typescript_1.default.SyntaxKind.QuestionToken; });
                                })), ", ").concat(typeString === "".concat(typeName, "[]"), "]")
                                : null;
                        })
                            .filter(Boolean);
                };
                results.push(methods
                    .map(function (m) {
                    var props = m.valueDeclaration
                        ? checker.getTypeOfSymbolAtLocation(m, m.valueDeclaration).getProperties()
                        : [];
                    var query = props.find(function (p) { return p.name === 'query'; });
                    var numberTypeQueryParams = query && getSomeTypeQueryParams_1('number', query);
                    var booleanTypeQueryParams = query && getSomeTypeQueryParams_1('boolean', query);
                    var validateInfo = [
                        { name: 'query', val: query },
                        { name: 'body', val: props.find(function (p) { return p.name === 'reqBody'; }) },
                        { name: 'headers', val: props.find(function (p) { return p.name === 'reqHeaders'; }) }
                    ]
                        .filter(function (prop) { return !!prop.val; })
                        .map(function (_a) {
                        var _b;
                        var name = _a.name, val = _a.val;
                        return ({
                            name: name,
                            type: val.valueDeclaration &&
                                checker.getTypeOfSymbolAtLocation(val, val.valueDeclaration),
                            hasQuestion: !!((_b = val.declarations) === null || _b === void 0 ? void 0 : _b.some(function (d) { return d.getChildAt(1).kind === typescript_1.default.SyntaxKind.QuestionToken; }))
                        });
                    })
                        .filter(function (_a) {
                        var type = _a.type;
                        return type === null || type === void 0 ? void 0 : type.isClass();
                    });
                    var reqFormat = props.find(function (p) { return p.name === 'reqFormat'; });
                    var isFormData = ((reqFormat === null || reqFormat === void 0 ? void 0 : reqFormat.valueDeclaration) &&
                        checker.typeToString(checker.getTypeOfSymbolAtLocation(reqFormat, reqFormat.valueDeclaration))) === 'FormData';
                    var reqBody = props.find(function (p) { return p.name === 'reqBody'; });
                    var hooksTexts = ['onRequest', 'preParsing', 'preValidation', 'preHandler']
                        .map(function (key) {
                        var _a, _b;
                        if (key === 'preValidation') {
                            var texts_1 = __spreadArray(__spreadArray(__spreadArray([
                                (numberTypeQueryParams === null || numberTypeQueryParams === void 0 ? void 0 : numberTypeQueryParams.length)
                                    ? ((_a = query === null || query === void 0 ? void 0 : query.declarations) === null || _a === void 0 ? void 0 : _a.some(function (d) { return d.getChildAt(1).kind === typescript_1.default.SyntaxKind.QuestionToken; }))
                                        ? "callParserIfExistsQuery(parseNumberTypeQueryParams([".concat(numberTypeQueryParams.join(', '), "]))")
                                        : "parseNumberTypeQueryParams([".concat(numberTypeQueryParams.join(', '), "])")
                                    : '',
                                (booleanTypeQueryParams === null || booleanTypeQueryParams === void 0 ? void 0 : booleanTypeQueryParams.length)
                                    ? ((_b = query === null || query === void 0 ? void 0 : query.declarations) === null || _b === void 0 ? void 0 : _b.some(function (d) { return d.getChildAt(1).kind === typescript_1.default.SyntaxKind.QuestionToken; }))
                                        ? "callParserIfExistsQuery(parseBooleanTypeQueryParams([".concat(booleanTypeQueryParams.join(', '), "]))")
                                        : "parseBooleanTypeQueryParams([".concat(booleanTypeQueryParams.join(', '), "])")
                                    : '',
                                isFormData && (reqBody === null || reqBody === void 0 ? void 0 : reqBody.valueDeclaration)
                                    ? "formatMultipartData([".concat(checker
                                        .getTypeOfSymbolAtLocation(reqBody, reqBody.valueDeclaration)
                                        .getProperties()
                                        .map(function (p) {
                                        var _a;
                                        var node = p.valueDeclaration &&
                                            checker.typeToTypeNode(checker.getTypeOfSymbolAtLocation(p, p.valueDeclaration), undefined, undefined);
                                        return node && (typescript_1.default.isArrayTypeNode(node) || typescript_1.default.isTupleTypeNode(node))
                                            ? "['".concat(p.name, "', ").concat(!!((_a = p.declarations) === null || _a === void 0 ? void 0 : _a.some(function (d) {
                                                return d
                                                    .getChildren()
                                                    .some(function (c) { return c.kind === typescript_1.default.SyntaxKind.QuestionToken; });
                                            })), "]")
                                            : undefined;
                                    })
                                        .filter(Boolean)
                                        .join(', '), "])")
                                    : ''
                            ], __read(genHookTexts_1('preValidation', m.name)), false), __read((query &&
                                __spreadArray(__spreadArray([], __read((numberTypeQueryParams !== null && numberTypeQueryParams !== void 0 ? numberTypeQueryParams : [])), false), __read((booleanTypeQueryParams !== null && booleanTypeQueryParams !== void 0 ? booleanTypeQueryParams : [])), false).some(function (t) { return t === null || t === void 0 ? void 0 : t.endsWith('true]'); }) &&
                                validateInfo.length
                                ? ['normalizeQuery']
                                : [])), false), [
                                validateInfo.length
                                    ? "createValidateHandler(req => [\n".concat(validateInfo
                                        .map(function (v) {
                                        return v.type
                                            ? "          ".concat(v.hasQuestion ? "Object.keys(req.".concat(v.name, " as any).length ? ") : '', "validateOrReject(plainToInstance(Validators.").concat(checker.typeToString(v.type), ", req.").concat(v.name, " as any, transformerOptions), validatorOptions)").concat(v.hasQuestion ? ' : null' : '')
                                            : '';
                                    })
                                        .join(',\n'), "\n        ])")
                                    : '',
                                dirPath.includes('@number')
                                    ? "createTypedParamsHandler(['".concat(dirPath
                                        .split('/')
                                        .filter(function (p) { return p.includes('@number'); })
                                        .map(function (p) { return p.split('@')[0].slice(1); })
                                        .join("', '"), "'])")
                                    : ''
                            ], false).filter(Boolean);
                            return texts_1.length
                                ? "".concat(key, ": ").concat(texts_1.length === 1
                                    ? texts_1[0].replace(/^\.+/, '')
                                    : "[\n        ".concat(texts_1.join(',\n        '), "\n      ]"))
                                : '';
                        }
                        var texts = genHookTexts_1(key, m.name).filter(Boolean);
                        return texts.length
                            ? "".concat(key, ": ").concat(texts.length === 1 ? texts[0].replace('...', '') : "[".concat(texts.join(', '), "]"))
                            : '';
                    })
                        .filter(Boolean);
                    return "  fastify.".concat(m.name, "(").concat(hooksTexts.length || (resSchemaMethods_1 === null || resSchemaMethods_1 === void 0 ? void 0 : resSchemaMethods_1.includes(m.name))
                        ? '\n    '
                        : '').concat(dirPath
                        ? "`${basePath}".concat("/".concat(dirPath)
                            .replace(/\/_/g, '/:')
                            .replace(/@.+?($|\/)/g, '$1'), "`")
                        : "basePath || '/'", ",").concat((function () {
                        var validatorsText = hasValidatorsMethods_1.includes(m.name)
                            ? "...validatorsToSchema(controller".concat(controllers.length, ".").concat(m.name, ".validators)")
                            : null;
                        var schemasText = hasSchemasMethods_1.includes(m.name)
                            ? "...controller".concat(controllers.length, ".").concat(m.name, ".schemas")
                            : null;
                        var paramsValidatorsText = paramsValidators.length
                            ? "params: ".concat(paramsValidators
                                .map(function (v) { return "".concat(v.name, ".params"); }
                            // v.isNumber ? `z.preprocess(Number, ${v.name}.params)` : `${v.name}.params`
                            )
                                .join('.and(')).concat(paramsValidators.length > 1 ? ')' : '')
                            : null;
                        var resSchemaText = (resSchemaMethods_1 === null || resSchemaMethods_1 === void 0 ? void 0 : resSchemaMethods_1.includes(m.name))
                            ? genResSchemaText_1(m.name)
                            : null;
                        var vals = __spreadArray(__spreadArray(__spreadArray([], __read((validatorsText && !schemasText && !resSchemaText && !paramsValidatorsText
                            ? ["schema: ".concat(validatorsText.slice(3))]
                            : schemasText && !validatorsText && !resSchemaText && !paramsValidatorsText
                                ? ["schema: ".concat(schemasText.slice(3))]
                                : validatorsText || schemasText || resSchemaText || paramsValidatorsText
                                    ? [
                                        "schema: {\n        ".concat([
                                            validatorsText,
                                            schemasText,
                                            resSchemaText,
                                            paramsValidatorsText
                                        ]
                                            .filter(Boolean)
                                            .join(',\n        '), "\n      }")
                                    ]
                                    : [])), false), __read((validatorsText || paramsValidatorsText ? ['validatorCompiler'] : [])), false), __read(hooksTexts), false);
                        return vals.length > 0
                            ? "\n    {\n      ".concat(vals.join(',\n      '), "\n    }").concat(fs_1.default.readFileSync("".concat(input, "/$relay.ts"), 'utf8').includes('AdditionalRequest')
                                ? ' as RouteShorthandOptions'
                                : '', ",")
                            : '';
                    })(), "\n    ").concat(isPromiseMethods_1.includes(m.name) ? 'asyncMethodToHandler' : 'methodToHandler', "(").concat("controller".concat(controllers.length, ".").concat(m.name).concat(hasHandlerMethods_1.includes(m.name) ? '.handler' : ''), ")").concat(hooksTexts.length || (resSchemaMethods_1 === null || resSchemaMethods_1 === void 0 ? void 0 : resSchemaMethods_1.includes(m.name))
                        ? '\n  '
                        : '', ")\n");
                })
                    .join('\n'));
                controllers.push(["".concat(input, "/controller"), !!ctrlHooksEvents_1, !!resSchemaMethods_1]);
            }
        }
        var childrenDirs = fs_1.default.readdirSync(input, { withFileTypes: true }).filter(function (d) { return d.isDirectory(); });
        if (childrenDirs.length) {
            results.push.apply(results, __spreadArray([], __read(childrenDirs
                .filter(function (d) { return !d.name.startsWith('_'); })
                .reduce(function (prev, d) { return __spreadArray(__spreadArray([], __read(prev), false), __read(createText(path_1.default.posix.join(dirPath, d.name), hooks, paramsValidators)), false); }, [])), false));
            var value = childrenDirs.find(function (d) { return d.name.startsWith('_'); });
            if (value) {
                results.push.apply(results, __spreadArray([], __read(createText(path_1.default.posix.join(dirPath, value.name), hooks, paramsValidators)), false));
            }
        }
        return results;
    };
    var text = createText('', [], []).join('\n');
    var ctrlHooks = controllers.filter(function (c) { return c[1]; });
    var resSchemas = controllers.filter(function (c) { return c[2]; });
    return {
        imports: "".concat(hooksPaths
            .map(function (m, i) {
            return "import hooksFn".concat(i, " from '").concat(m.replace(/^api/, './api').replace(appDir, './api'), "'\n");
        })
            .join('')).concat(validatorsPaths
            .map(function (m, i) {
            return "import validatorsFn".concat(i, " from '").concat(m.replace(/^api/, './api').replace(appDir, './api'), "'\n");
        })
            .join('')).concat(controllers
            .map(function (ctrl, i) {
            return "import controllerFn".concat(i).concat(ctrl[1] || ctrl[2]
                ? ", { ".concat(ctrl[1] ? "hooks as ctrlHooksFn".concat(ctrlHooks.indexOf(ctrl)) : '').concat(ctrl[1] && ctrl[2] ? ', ' : '').concat(ctrl[2] ? "responseSchema as responseSchemaFn".concat(resSchemas.indexOf(ctrl)) : '', " }")
                : '', " from '").concat(ctrl[0].replace(/^api/, './api').replace(appDir, './api'), "'\n");
        })
            .join('')),
        consts: "".concat(hooksPaths
            .map(function (_, i) { return "  const hooks".concat(i, " = hooksFn").concat(i, "(fastify)\n"); })
            .join('')).concat(ctrlHooks
            .map(function (_, i) { return "  const ctrlHooks".concat(i, " = ctrlHooksFn").concat(i, "(fastify)\n"); })
            .join('')).concat(validatorsPaths
            .map(function (_, i) { return "  const validators".concat(i, " = validatorsFn").concat(i, "(fastify)\n"); })
            .join('')).concat(resSchemas
            .map(function (_, i) { return "  const responseSchema".concat(i, " = responseSchemaFn").concat(i, "()\n"); })
            .join('')).concat(controllers
            .map(function (_, i) { return "  const controller".concat(i, " = controllerFn").concat(i, "(fastify)\n"); })
            .join('')),
        controllers: text
    };
});
//# sourceMappingURL=createControllersText.js.map